@@ -337,9 +337,32 @@ sub decode_body {
     
     my $content_type = $request->{ds}{backend_plugin}->get_content_type() // '';
     
+    # If the content type is application/x-www-form-urlencoded, then it was already unpacked into
+    # a hash ref.
+    
+    if ( $content_type =~ qr{^application/x-www-form-urlencoded}xsi )
+    {
+	# my @chunks = split(/&/, $request->{raw_body});
+	# $request->{decoded_body} = { };
+	
+	# foreach my $chunk ( @chunks )
+	# {
+	#     my ($var, $value) = split(/=/, $chunk, 2);
+
+	#     $var =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
+	#     $value =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
+
+	#     $request->{decoded_body}{$var} = $value;
+	# }
+	
+	$request->{decoded_body} = $request->{raw_body};
+	
+	return $request->{decoded_body};
+    }
+    
     # If the body starts and ends with '{' or '[', assume the format is JSON regardless of content type.
     
-    if ( $request->{raw_body} =~ / ^ [{] .* [}] $ | ^ [\[] .* [\]] $ /xsi )
+    elsif ( $request->{raw_body} =~ / ^ [{] .* [}] $ | ^ [\[] .* [\]] $ /xsi )
     {
 	try {
 	    unless ( defined $request->{decoded_body} )
@@ -360,26 +383,6 @@ sub decode_body {
 	return ($request->{decoded_body}, $request->{body_error});
     }
     
-    # If the content type is application/x-www-form-urlencoded, then unpack it as a single record.
-
-    elsif ( $content_type =~ qr{^application/x-www-form-urlencoded}xsi )
-    {
-	my @chunks = split(/&/, $request->{raw_body});
-	$request->{decoded_body} = { };
-	
-	foreach my $chunk ( @chunks )
-	{
-	    my ($var, $value) = split(/=/, $chunk, 2);
-
-	    $var =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
-	    $value =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/eg;
-
-	    $request->{decoded_body}{$var} = $value;
-	}
-	
-	return $request->{decoded_body};
-    }
-    
     # Otherwise, split into rows and return.
     
     else
@@ -854,6 +857,24 @@ sub result_counts {
 }
 
 
+# set_extra_datainfo ( key, value )
+# 
+# Register a key, with a corresponding value. This key/value pair will be added to the datainfo
+# list, and presented if the user has asked for it to be displayed.  If the output format is JSON,
+# the value may be a hashref or arrayref. Otherwise, it should be a scalar. If the value is
+# undefined, nothing will be displayed, and any previously set value for this key will be
+# removed. Keys will be displayed in the order in which they were first set.
+
+sub set_extra_datainfo {
+    
+    my ($request, $key, $title, $value) = @_;
+    
+    $request->{extra_datainfo}{$key} = $value;
+    $request->{title_datainfo}{$key} = $title;
+    push @{$request->{list_datainfo}}, $key;
+}
+
+
 # linebreak
 # 
 # Return the linebreak sequence that should be used for the output of this request.
@@ -1101,9 +1122,46 @@ sub clear_result {
 
 sub skip_output_record {
 
-    my ($request) = @_;
+    my ($request, $record) = @_;
     
-    $request->{_skip_record} = 1;
+    $record->{_skip_record} = 1 if $record;
+}
+
+
+# alternate_output_block ( block_name )
+# 
+# Call this method from a before_record_hook routine to select an alternate output block for the
+# record.
+
+sub alternate_output_block {
+
+    my ($request, $block_name) = @_;
+
+    croak "'alternate_output_block' is obsolete.";
+    
+    # croak "unknown block '$block_name'" unless $request->{ds}{block}{$block_name};
+    
+    # $request->{_alternate_block} = $block_name;
+
+    # unless ( exists $request->{block_field_list}{$block_name} )
+    # {
+    # 	$request->{ds}->configure_block($request, $block_name);
+    # }
+}
+
+
+sub select_output_block {
+    
+    my ($request, $record, $block_name) = @_;
+    
+    croak "unknown block '$block_name'" unless $request->{ds}{block}{$block_name};
+    
+    $record->{_output_block} = $block_name;
+
+    unless ( exists $request->{block_field_list}{$block_name} )
+    {
+	$request->{ds}->configure_block($request, $block_name);
+    }
 }
 
 
