@@ -844,6 +844,18 @@ sub configure_block {
 		    $output->{$2} = $r->{$key} if $vocab eq $1;
 		}
 		
+		elsif ( $key eq 'data_type' )
+		{
+		    my $type_value = $r->{data_type};
+		    croak "unknown value '$r->{data_type}' for data_type: must be one of 'int', 'pos', 'dec', 'str'"
+			unless lc $type_value eq 'int' || lc $type_value eq 'pos' ||
+			    lc $type_value eq 'dec' || lc $type_value eq 'str';
+		    
+		    $output->{data_type} = $r->{data_type};
+		    push @{$request->{proc_list}}, { check_field => $r->{output}, data_type => $r->{data_type} }
+			unless $r->{data_type} eq 'str';
+		}
+		
 		elsif ( $key ne 'output' )
 		{
 		    warn "Warning: unknown key '$key' in output record\n";
@@ -1869,7 +1881,6 @@ sub _generate_single_result {
 	unless $format_class;
     
     my $path = $request->node_path;
-    my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
     
     # Set the result count to 1, in case the client asked for it.
     
@@ -1888,17 +1899,15 @@ sub _generate_single_result {
 	$request->add_warning("No output fields were defined for this request.");
     }
     
-    # If there are any processing steps to do, then do them.
-    
-    $ds->process_record($request, $request->{main_record}, $proc_list);
-    
     # If there is a before_record_hook defined for this path, call it now. For a single result,
     # calls to 'skip_output_record' are not allowed.
     
-    if ( $ds->{hook_enabled}{before_record_hook} )
-    {
-	$ds->_call_hooks($path, 'before_record_hook', $request, $request->{main_record})
-    }
+    $ds->_call_hooks($request, 'before_record_hook', $request->{main_record})
+	if $request->{hook_enabled}{before_record_hook};
+    
+    # If there are any processing steps to do, then do them.
+    
+    $ds->process_record($request, $request->{main_record}, $proc_list);
     
     # Generate the initial part of the output, before the first record.
     
@@ -1914,13 +1923,13 @@ sub _generate_single_result {
 
     # If an after_serialize_hook is defined for this path, call it.
     
-    if ( $serial_hook )
+    if ( $request->{hook_enabled}{after_serialize_hook} )
     {
 	my $rs = '';
 	
-	$ds->_call_hook_list($serial_hook, $request, 'header', \$header);
-	$ds->_call_hook_list($serial_hook, $request, 'record', \$rs, \$record);
-	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+	$ds->_call_hooks($request, 'after_serialize_hook', 'header', \$header);
+	$ds->_call_hooks($request, 'after_serialize_hook', 'record', \$rs, \$record);
+	$ds->_call_hooks($request, 'after_serialize_hook', 'footer', \$footer);
     }
     
     return $header . $record . $footer;
@@ -1951,12 +1960,6 @@ sub _generate_compound_result {
     my $path = $request->node_path;
     my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
     
-    # Get the lists that specify how to process each record and which fields
-    # to output.
-    
-    my $proc_list = $request->{proc_list};
-    my $field_list = $request->{field_list};
-    
     # If we have an explicit result list, then we know the count.
     
     $request->{result_count} = scalar(@{$request->{main_result}})
@@ -1964,11 +1967,11 @@ sub _generate_compound_result {
     
     # Generate the initial part of the output, before the first record.
     
-    my $output = $format_class->emit_header($request, $field_list);
+    my $output = $format_class->emit_header($request, $request->{field_list});
     
-    if ( $serial_hook )
+    if ( $request->{hook_enabled}{after_serialize_hook} )
     {
-	$ds->_call_hook_list($serial_hook, $request, 'header', \$output);	
+	$ds->_call_hooks($request, 'after_serialize_hook', 'header', \$output);	
     }
     
     # A record separator is emitted before every record except the first.  If
@@ -2009,29 +2012,41 @@ sub _generate_compound_result {
  RECORD:
     while ( my $record = $ds->_next_record($request) )
     {
+	my $proc_list = $request->{proc_list};
+	my $field_list = $request->{field_list};
+	
+	# If there is a before_record_hook defined for this path, call it now.
+	
+	if ( $request->{hook_enabled}{before_record_hook} )
+	{
+	    $ds->_call_hooks($request, 'before_record_hook', $record);
+	}
+
+	# If 'skip_output_record' was called on this record, then skip it now. If
+	# 'select_output_block' was called, then substitute the field list and proc list associated
+	# with that block.
+
+	next RECORD if $record->{_skip_record};
+	
+	if ( my $alt = $record->{_output_block} )
+	{
+	    $proc_list = $request->{block_proc_list}{$alt};
+	    $field_list = $request->{block_field_list}{$alt};
+	}
+	
 	# If there are any processing steps to do, then process this record.
 	
 	$ds->process_record($request, $record, $proc_list);
 	
-	# If there is a before_record_hook defined for this path, call it now.
-	# If it calls 'skip_output_record', then do not output this record.
-	
-	if ( $ds->{hook_enabled}{before_record_hook} )
-	{
-	    delete $request->{_skip_record};
-	    $ds->_call_hooks('before_record_hook', $request, $record);
-	    next RECORD if $request->{_skip_record};
-	}
-	
 	# Generate the output for this record, preceded by a record separator if
 	# it is not the first record.
 	
 	my $outrs = $emit_rs ? $request->{rs} : ''; $emit_rs = 1;
 	my $outrec = $format_class->emit_record($request, $record, $field_list);
 	
-	if ( $serial_hook )
+	if ( $request->{hook_enabled}{after_serialize_hook} )
 	{
-	    $ds->_call_hook_list($serial_hook, $request, 'record', \$outrs, \$outrec);
+	    $ds->_call_hooks($request, 'after_serialize_hook', 'record', \$outrs, \$outrec);
 	}
 	
 	$output .= $outrs . $outrec;
@@ -2069,9 +2084,9 @@ sub _generate_compound_result {
     {
 	my $empty = $format_class->emit_empty($request);
 	
-	if ( $serial_hook )
+	if ( $request->{hook_enabled}{after_serialize_hook} )
 	{
-	    $ds->_call_hook_list($serial_hook, $request, 'empty', \$empty);
+	    $ds->_call_hooks($request, 'after_serialize_hook', 'empty', \$empty);
 	}
 
 	$output .= $empty;
@@ -2079,11 +2094,11 @@ sub _generate_compound_result {
     
     # Generate the final part of the output, after the last record.
     
-    my $footer = $format_class->emit_footer($request, $field_list);
+    my $footer = $format_class->emit_footer($request, $request->{field_list});
 
-    if ( $serial_hook )
+    if ( $request->{hook_enabled}{after_serialize_hook} )
     {
-	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+	$ds->_call_hooks($request, 'after_serialize_hook', 'footer', \$footer);
     }
     
     $output .= $footer;
@@ -2133,12 +2148,6 @@ sub _generate_processed_result {
     my $path = $request->node_path;
     my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
     
-    # Get the lists that specify how to process each record and which fields
-    # to output.
-    
-    my $proc_list = $request->{proc_list};
-    my $field_list = $request->{field_list};
-    
     # Now fetch and process each output record in turn.  Collect up all of the
     # records that pass the processing phase in a list.
     
@@ -2147,20 +2156,29 @@ sub _generate_processed_result {
  RECORD:
     while ( my $record = $ds->_next_record($request) )
     {
-	# If there are any processing steps to do, then process this record.
-	# If the return value is not true, skip the record.
-	
-	$ds->process_record($request, $record, $proc_list) or next RECORD;
+	my $proc_list = $request->{proc_list};
+	my $field_list = $request->{field_list};
 	
 	# If there is a before_record_hook defined for this path, call it now.
-	# If it calls 'skip_output_record', then do not output this record.
 	
-	if ( $ds->{hook_enabled}{before_record_hook} )
+	if ( $request->{hook_enabled}{before_record_hook} )
 	{
-	    delete $request->{_skip_record};
-	    $ds->_call_hooks('before_record_hook', $request, $record);
-	    next RECORD if $request->{_skip_record};
+	    $ds->_call_hooks($request, 'before_record_hook', $record);
 	}
+
+	# If 'skip_output_record' was called on this record, skip it now. If 'select_output_block'
+	# was called, then substitute the proc list associated with the selected output block.
+	
+	next RECORD if $record->{_skip_record};
+	
+	if ( $record->{_output_block} )
+	{
+	    $proc_list = $request->{block_proc_list}{$record->{_output_block}};
+	}
+	
+	# If there are any processing steps to do, then process this record.
+	
+	$ds->process_record($request, $record, $proc_list);
 	
 	# Add the record to the list.
 	
@@ -2173,11 +2191,11 @@ sub _generate_processed_result {
     
     # At this point, we can generate the output.  We start with the header.
     
-    my $output = $format_class->emit_header($request, $field_list);
+    my $output = $format_class->emit_header($request, $request->{field_list});
     
-    if ( $serial_hook )
+    if ( $request->{hook_enabled}{after_serialize_hook} )
     {
-	$ds->_call_hook_list($serial_hook, $request, 'header', \$output);	
+	$ds->_call_hooks($request, 'after_serialize_hook', 'header', \$output);	
     }
     
     # A record separator is emitted before every record except the first.  If
@@ -2213,16 +2231,21 @@ sub _generate_processed_result {
     while ( @results )
     {
 	my $record = shift @results;
+
+	# If an alternate block was specified for this record, use it.
+
+	my $field_list = $record->{block_field_list}{$record->{_output_block}}
+	    if $record->{_output_block};
 	
 	# Generate the output for this record, preceded by a record separator if
 	# it is not the first record.
 	
 	my $outrs = $emit_rs ? $request->{rs} : ''; $emit_rs = 1;
-	my $outrec = $format_class->emit_record($request, $record, $field_list);
+	my $outrec = $format_class->emit_record($request, $record, $field_list || $request->{field_list});
 	
-	if ( $serial_hook )
+	if ( $request->{hook_enabled}{after_serialize_hook} )
 	{
-	    $ds->_call_hook_list($serial_hook, $request, 'record', \$outrs, \$outrec);
+	    $ds->_call_hooks($request, 'after_serialize_hook', 'record', \$outrs, \$outrec);
 	}
 	
 	$output .= $outrs . $outrec;
@@ -2262,21 +2285,21 @@ sub _generate_processed_result {
     {
 	my $empty = $format_class->emit_empty($request);
 	
-	if ( $serial_hook )
+	if ( $request->{hook_enabled}{after_serialize_hook} )
 	{
-	    $ds->_call_hook_list($serial_hook, $request, 'empty', \$empty);
+	    $ds->_call_hooks($request, 'after_serialize_hook', 'empty', \$empty);
 	}
-
+	
 	$output .= $empty;
     }
     
     # Generate the final part of the output, after the last record.
     
-    my $footer = $format_class->emit_footer($request, $field_list);
+    my $footer = $format_class->emit_footer($request, $request->{field_list});
     
-    if ( $serial_hook )
+    if ( $request->{hook_enabled}{after_serialize_hook} )
     {
-	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+	$ds->_call_hooks($request, 'after_serialize_hook', 'footer', \$footer);
     }
     
     $output .= $footer;
@@ -2331,12 +2354,6 @@ sub _stream_compound_result {
     my $path = $request->node_path;
     my $serial_hook = $ds->{hook_enabled}{after_serialize_hook} && $ds->node_attr($path, 'after_serialize_hook');
     
-    # Get the lists that specify how to process each record and which fields
-    # to output.
-    
-    my $proc_list = $request->{proc_list};
-    my $field_list = $request->{field_list};
-    
     # Determine the output character set, because we will need to encode text
     # responses in it.
     
@@ -2362,22 +2379,42 @@ sub _stream_compound_result {
   RECORD:
     while ( my $record = $ds->_next_record($request) )
     {
+	my $proc_list = $request->{proc_list};
+	my $field_list = $request->{field_list};
+	
 	# If there are any processing steps to do, then process this record. But skip this if this
 	# subroutine was called from '_generate_processed_result'.
 
-	unless ( $request->{processing_complete} )
+	if ( $request->{processing_complete} )
+	{
+	    $field_list = $record->{block_field_list}{$record->{_output_block}}
+		if $record->{_output_block};
+	}
+	
+	else
 	{
-	    $ds->process_record($request, $record, $proc_list);
-	    
 	    # If there is a before_record_hook defined for this path, call it now.
-	    # If it calls 'skip_output_record', then do not output this record.
 	    
-	    if ( $ds->{hook_enabled}{before_record_hook} )
+	    if ( $request->{hook_enabled}{before_record_hook} )
 	    {
-		delete $request->{_skip_record};
-		$ds->_call_hooks('before_record_hook', $request, $record);
-		next RECORD if $request->{_skip_record};
+		$ds->_call_hooks($request, 'before_record_hook', $record);
 	    }
+
+	    # If 'skip_output_record' was called on this record, skip it now. If
+	    # 'select_output_block' was called, then substitute the proc list and field list for
+	    # the selected block.
+	    
+	    next RECORD if $record->{_skip_record};
+	    
+	    if ( my $alt = $record->{_output_block} )
+	    {
+		$proc_list = $request->{block_proc_list}{$alt};
+		$field_list = $request->{block_field_list}{$alt};
+	    }
+	    
+	    # Do any processing steps that were defined for this record.
+	    
+	    $ds->process_record($request, $record, $proc_list);
 	}
 	
 	# Generate the output for this record, preceded by a record separator if
@@ -2386,9 +2423,9 @@ sub _stream_compound_result {
 	my $outrs = $request->{rs};
 	my $outrec = $format_class->emit_record($request, $record, $field_list);
 	
-	if ( $serial_hook )
+	if ( $request->{hook_enabled}{after_serialize_hook} )
 	{
-	    $ds->_call_hook_list($serial_hook, $request, 'record', \$outrs, \$outrec);
+	    $ds->_call_hooks($request, 'after_serialize_hook', 'record', \$outrs, \$outrec);
 	}
 	
 	my $output .= $outrs . $outrec;
@@ -2424,11 +2461,11 @@ sub _stream_compound_result {
     
     # Generate the final part of the output, after the last record.
     
-    my $footer = $format_class->emit_footer($request, $field_list);
+    my $footer = $format_class->emit_footer($request, $request->{field_list});
     
-    if ( $serial_hook )
+    if ( $request->{hook_enabled}{after_serialize_hook} )
     {
-	$ds->_call_hook_list($serial_hook, $request, 'footer', \$footer);
+	$ds->_call_hooks($request, 'after_serialize_hook', 'footer', \$footer);
     }
     
     if ( ! defined $footer or $footer eq '' )
