@@ -33,9 +33,9 @@ sub emit_header {
     
     if ( $request->display_datainfo )
     {
-	my $info = $request->data_info;
+	my $info = $request->datainfo;
 	
-	foreach my $key ( $request->data_info_keys )
+	foreach my $key ( $request->datainfo_keys )
 	{
 	    next unless $info->{$key};
 	    my $value = json_clean($info->{$key});
@@ -43,14 +43,6 @@ sub emit_header {
 	    $output .= qq{"$key":$value,\n};
 	}
 	
-	#$output .= '"data_provider":' . json_clean($info->{data_provider}) . ",\n";
-	#$output .= '"data_source":' . json_clean($info->{data_source}) . ",\n";
-	#$output .= '"data_source_url":' . json_clean($base . '/') . ",\n";
-	#$output .= '"data_license":' . json_clean($info->{license}) . ",\n";
-	#$output .= '"data_license_url":' . json_clean($info->{license_url}) . ",\n";
-	#$output .= '"documentation_url":' . json_clean($doc_url) . ",\n";
-	#$output .= '"data_url":' . json_clean($data_url) . ",\n";
-	#$output .= '"access_time":' . json_clean($info->{access_time}) . ",\n";
 	$output .= '"parameters":{' . "\n";
 	
 	my @display = $request->params_for_display;
@@ -73,11 +65,43 @@ sub emit_header {
 	    
 	    else
 	    {
-		$output .= qq<"$param":"$value">;
+		$value = json_clean($value);
+		$output .= qq<"$param":$value>;
 	    }
 	}
 	
 	$output .= "\n},\n";
+
+	# If there have been any extra datainfo fields set, list them now.
+
+	if ( my @extra = $request->extra_datainfo_keys )
+	{
+	    foreach my $key ( @extra )
+	    {
+		next unless defined $key;
+		my $value = $request->extra_datainfo($key);
+		next unless defined $value;
+		
+		$output .= $sep; $sep = ",\n";
+		
+		if ( ref $value && reftype $value eq 'HASH' )
+		{
+		    # not implemented yet
+		}
+
+		elsif ( ref $value && reftype $value eq 'ARRAY' )
+		{
+		    $value = $class->emit_array($request, $value);
+		}
+
+		else
+		{
+		    $value = json_clean($value);
+		}
+		
+		$output .= qq<"$key":$value,\n>;
+	    }
+	}
     }
     
     # Check if we have been asked to report the result count, and if it is
