@@ -43,6 +43,11 @@ sub define_ruleset {
     croak "define_ruleset: invalid ruleset name '$ruleset_name'\n"
 	unless $ruleset_name =~ qr{ ^ [\w.:][\w.:-]* $ }xs;
     
+    # If we are in 'diagnostic' mode, then we are generating diagnostics rather than
+    # output.  So keep extra attributes and don't send them to HTTP::Validate.
+    
+    my $diag_mode = $ds->is_mode('diagnostic');
+    
     # Go through the arguments one by one, to see which ones (if any) need to
     # be edited before passing them on to HTTP::Validate.
     
@@ -58,30 +63,28 @@ sub define_ruleset {
     {
 	next unless defined $arg;
 	
-	# Documentation strings are just passed on through.
+	# Documentation strings that do not start with >> are just added to
+	# the documentation for the current rule (or are added to the
+	# beginning of the documentation if there is no current rule).
 	
-	unless ( ref $arg )
+	if ( ! ref $arg && $arg !~ qr{^>>}s )
 	{
 	    push @pending_doc, $arg;
 	    next ARG;
 	}
 	
-	# Any reference that is not a hashref should be flagged.
-	
-	if ( ref $arg ne 'HASH' )
-	{
-	    croak "define_ruleset: the arguments must be a list of hashrefs and strings\n";
-	}
-	
-	# Otherwise, we have a new rule that needs to be checked.  So first
-	# take care of any pending rule and its documentation.
+	# Otherwise, we have a new top-level item (either a rule or a
+	# documentation paragraph.  So if there is a pending rule then finish
+	# it now.
 	
 	if ( $pending_rule )
 	{
-	    push @rules_and_doc, $pending_rule, @pending_doc, @final_doc;
+	    if ( $ds->check_rule($pending_rule, \@pending_doc) )
+	    {
+		push @rules_and_doc, $pending_rule, @pending_doc, @final_doc;
+	    }
+	    
 	    $pending_rule = undef;
-	    @pending_doc = ();
-	    @final_doc = ();
 	}
 	
 	# If there is just pending documentation, then add it.
@@ -89,16 +92,52 @@ sub define_ruleset {
 	elsif ( @pending_doc )
 	{
 	    push @rules_and_doc, @pending_doc;
-	    @pending_doc = ();
-	    @final_doc = ();
 	}
 	
-	# Now we consider the new rule.
+	# Then clear the "pending" lists.
+	
+	@pending_doc = ();
+	@final_doc = ();
+	
+	# Now, if we have a documentation string starting with ">>", add it.
+	
+	if ( ! ref $arg )
+	{
+	    push @rules_and_doc, $arg;
+	    next ARG;
+	}
+	
+	# Any reference that is not a hashref should be flagged.
+	
+	elsif ( ref $arg ne 'HASH' )
+	{
+	    croak "define_ruleset: the arguments must be a list of hashrefs and strings\n";
+	}
+	
+	# If we get here, then $arg must be a new rule.  If it is a substitution
+	# rule, then move its value to the RS_SUBST hash and otherwise ignore
+	# it.  This is a feature implemented by Web::DataService and not by
+	# HTTP::Validate, so we cannot pass this rule to the latter module.
+	
+	if ( $arg->{substitute} )
+	{
+	    croak "define_ruleset: the value of 'substitute' must be a hashref\n"
+		unless ref $arg->{substitute} eq 'HASH';
+	    
+	    croak "define_ruleset: unknown key in 'subtitute' rule\n"
+		if keys %$arg > 1;
+	    
+	    $ds->{RS_SUBST}{$ruleset_name} = $arg->{substitute};
+	    
+	    next ARG;
+	}
+	
+	# Otherwise, we make this the "pending rule" for subsequent documentation.
 	
 	$pending_rule = $arg;
 	
-	# Look at the rule type attribute and its value.  Rules that are not
-	# parameter rules are simply passed through.
+	# We then check for an attribute that specifies the rule type.  Rules that
+	# are not parameter rules are simply passed through to HTTP::Validate as-is.
 	
 	my $ruletype = $arg->{optional}  ? 'optional' 
 		     : $arg->{param}     ? 'param'
@@ -107,6 +146,8 @@ sub define_ruleset {
 	
 	next ARG unless $ruletype;
 	
+	# The value of the rule-type attribute is the parameter name.
+	
 	my $param = $arg->{$ruletype};
 	
 	# Now look at the value(s) specified by the 'valid' attribute.
@@ -135,8 +176,12 @@ sub define_ruleset {
 	    # documentation string to the end of the documentation for this
 	    # rule, unless the attribute 'no_set_doc' was also specified.
 	    
+	    # If we are in diagnostic mode, save the actual set name so it can
+	    # be printed out later.
+	    
 	    elsif ( $ds->set_defined($v) )
 	    {
+		$arg->{valid_save} = $arg->{valid} if $diag_mode;
 		$arg->{valid} = $ds->valid_set($v);
 		
 		unless ( $arg->{no_set_doc} )
@@ -214,14 +259,12 @@ sub define_ruleset {
 		{
 		    $arg->{$ruletype} = $ds->{special}{show};
 		    $arg->{list} = ',';
-		    @pending_doc = $ds->generate_special_doc($special_arg)
-			unless @pending_doc;
+		    $arg->{special} = 'show';
 		}
 		
 		else
 		{
-		    $pending_rule = undef;
-		    next ARG;
+		    $arg->{special} = 'IGNORE';
 		}
 	    }
 	    
@@ -239,21 +282,21 @@ sub define_ruleset {
 		
 		unless ( $ds->{special}{$special_arg} )
 		{
-		    $pending_rule = undef;
+		    $pending_rule = { special => 'IGNORE' };
 		    next ARG;
 		}
 		
 		$pending_rule = $ds->generate_special_rule($special_arg);
-		@pending_doc = $ds->generate_special_doc($special_arg)
-		    unless @pending_doc;
+		$pending_rule->{special} = $special_arg;
 		
 		# If the original rule specified any of 'errmsg', 'warn', 
 		# 'alias', or 'clean', copy these over to the new rule.
 		
 		$pending_rule->{errmsg} = $arg->{errmsg} if defined $arg->{errmsg};
 		$pending_rule->{warn} = $arg->{warn} if defined $arg->{warn};
-		$pending_rule->{alias} = $arg->{alias} if defined $arg->{alias};
 		$pending_rule->{clean} = $arg->{clean} if defined $arg->{clean};
+		$pending_rule->{alias} = $arg->{alias} if defined $arg->{alias} &&
+		    $arg->{alias} ne $pending_rule->{optional};
 		
 		# Mark that this parameter has already been dealt with, so
 		# that a later rule with 'all' will not include it a second
@@ -278,10 +321,52 @@ sub define_ruleset {
     
     if ( $pending_rule )
     {
-	push @rules_and_doc, $pending_rule, @pending_doc, @final_doc;
+	if ( $ds->check_rule($pending_rule, \@pending_doc) )
+	{
+	    push @rules_and_doc, $pending_rule, @pending_doc, @final_doc;
+	}
     }
     
-    # Now call HTTP::Validate::define_ruleset.  Wrap it in a 'eval' block so
+    # If we are in 'diagnostic' mode, then stash a copy of the ruleset
+    # definition where it can be printed out later.
+    
+    if ( $diag_mode )
+    {
+	my @diag_copy;
+	
+	# Go through each of the entries in the ruleset definition
+	
+	foreach my $r ( @rules_and_doc )
+	{
+	    # If this is a rule definition, then save a copy.  Rename the key
+	    # 'valid_save' to 'valid' in the copy, and delete 'valid_save'
+	    # from the real definition if it exists so that it won't cause
+	    # HTTP::Validate to throw an error.  This is necessary because a
+	    # code reference is not useful in the diagnostic data structure
+	    # but the name of the set is.
+	    
+	    if ( ref $r eq 'HASH' )
+	    {
+		my $copy = { %$r };
+		$copy->{valid} = $copy->{valid_save} if $copy->{valid_save};
+		delete $copy->{valid_save};
+		delete $r->{valid_save};
+		
+		push @diag_copy, $copy;
+	    }
+	    
+	    # Documentation strings are just passed right through.
+	    
+	    else
+	    {
+		push @diag_copy, $r;
+	    }
+	}
+	
+	$ds->{ruleset_diag}{$ruleset_name} = \@diag_copy;
+    }
+    
+    # Then call HTTP::Validate::define_ruleset.  Wrap it in a 'eval' block so
     # that we can catch any errors and pass them to 'croak'.
     
     my $error_msg;
@@ -299,6 +384,26 @@ sub define_ruleset {
 }
 
 
+# check_rule ( rr, doc_ref )
+# 
+# Check and adjust the special rule and its documentation.  Return true if
+# this rule should be included in the ruleset, false otherwise.
+
+sub check_rule {
+    
+    my ($ds, $rr, $doc_ref) = @_;
+    
+    return 1 unless $rr->{special};
+    return 0 if $rr->{special} eq 'IGNORE';
+    
+    @$doc_ref = $ds->generate_special_doc($rr->{special})
+	unless @$doc_ref;
+    
+    delete $rr->{special};
+    return 1;
+}
+
+
 # generate_special_rule ( param )
 # 
 # Generate a rule for the given special parameter.
@@ -335,7 +440,7 @@ sub generate_special_rule {
 	$rule->{valid} = FLAG_VALUE;
     }
     elsif ( $param eq 'linebreak' ) {
-	$rule->{valid} = ENUM_VALUE('cr', 'crlf');
+	$rule->{valid} = ENUM_VALUE('cr', 'lf', 'crlf');
     }
     elsif ( $param eq 'vocab' ) {
 	$rule->{valid} = $ds->valid_vocab;
@@ -517,4 +622,31 @@ sub list_ruleset_params {
 }
 
 
+sub document_ruleset {
+    
+    my ($ds, $rs_name) = @_;
+    
+    my $doc = $ds->validator->document_params($rs_name);
+    
+    if ( ref $ds->{RS_SUBST}{$rs_name} eq 'HASH' )
+    {
+	$ds->do_doc_substitution(\$doc, $rs_name);
+    }
+    
+    return $doc;
+}
+
+
+sub do_doc_substitution {
+    
+    my ($ds, $doc_ref, $rs_name) = @_;
+    
+    my $subst = $ds->{RS_SUBST}{$rs_name};
+    
+    my $pattern = join('|', keys %$subst);
+    
+    $$doc_ref =~ s{($pattern)}{$subst->{$1}}egm;
+}
+
+
 1;
